require 'rspec_failsafe_formatter/TeeIO'
require 'rspec/core/formatters'
require 'rspec/core/formatters/console_codes'
require 'nokogiri'
require 'fileutils'

class RspecFailsafeFormatter
  RSpec::Core::Formatters.register self, :example_passed,
                                   :example_pending,
                                   :example_failed,
                                   :example_group_started,
                                   :example_group_finished,
                                   :dump_summary,
                                   :seed,
                                   :close

  attr_reader :output, :failed_notifications

  def initialize(output)
    FileUtils::mkdir_p 'target/failsafe-reports'
    @out_file = File.new('target/failsafe-reports/rspec.txt', 'w')
    @output = TeeIO.new(output, @out_file)
    @group_level = 0
    @index_offset = 0
    @failed_notifications = []
  end

  Color = ::RSpec::Core::Formatters::ConsoleCodes

  def example_passed(notification)
    output.print Color.wrap('.', :success)
  end

  def example_pending(notification)
    output.print Color.wrap('*', :pending)
  end

  def example_failed(notification)
    @failed_notifications << notification
    output.print Color.wrap('F', :failure)
  end

  def example_group_started(event)
    if @group_level.zero?
      output.print "#{event.group.description} "
      @start_time = Time.now
    end

    @group_level += 1
  end

  def example_group_finished(event)
    @group_level -= 1

    if @group_level.zero?
      failed_notifications.each_with_index do |failure, index|
        output.puts failure.fully_formatted(@index_offset + index + 1)
      end

      @index_offset += failed_notifications.size
      failed_notifications.clear
    end
  end

  def dump_summary notification # SummaryNotification
    xml = Nokogiri::XML::Builder.new(:encoding => 'UTF-8') do |xml|
      xml.send(:'failsafe-summary', timeout: false) {
        xml.completed notification.example_count
        xml.errors 0
        xml.failures notification.failure_count
        xml.skipped notification.pending_count
        if notification.failure_count > 0
          xml.failureMessage notification.failed_examples.first.exception.to_s
        else
          xml.failureMessage
        end
      }
    end

    file = File.new('target/failsafe-reports/failsafe-summary-rspec.xml', 'wb')
    file.write(xml.to_xml)
    file.close
    @output << "\n\nFinished in #{notification.duration.round(4)} seconds.\n\n"
  end

  def seed(notification)
    return unless notification.seed_used?
    output.puts notification.fully_formatted
  end

  def close(notification)
    @out_file.close
  end

end